#include "wifi_access_point.h"

char data_sentence_buffer[1024];
WifiInterface* wifi = new WifiAccessPoint();

unsigned long airdata_count = 0;

void measurements_read_and_send() {
  sprintf(data_sentence_buffer,
	  "$AR,%ld,%10.6f,%10.6f,%10.6f,%10.6f,%10.6f,%ld",
	  airdata_count,
	  25,
    30,
    40,
    50,
    60,
    millis());
  wifi->send(data_sentence_buffer);
  airdata_count++;
}

hw_timer_t *timer = NULL;
portMUX_TYPE timer_mux = portMUX_INITIALIZER_UNLOCKED;
bool timer_start_measurement = false;

void timer_fired() {
  portENTER_CRITICAL(&timer_mux);
  timer_start_measurement = true;
  portEXIT_CRITICAL(&timer_mux);
}

// Speed of the I2C bus
#define I2C_BUS_SPEED 400000L // 400 kHz

// How frequently (in uS) should measurements be taken?
#define MEASUREMENT_INTERVAL_US 50000 // 50 ms = 20 Hz

void setup() {
  Serial.begin(115200);
  wifi->begin();
  // Use 1st timer of 4 (counted from zero).
  // Set 80 divider for prescaler (see ESP32 Technical Reference Manual
  // for more info).
  timer = timerBegin(0, 80, true);
  timerAttachInterrupt(timer, &timer_fired, true);
  timerAlarmWrite(timer, MEASUREMENT_INTERVAL_US, true);
  timerAlarmEnable(timer);
}

void loop() {
  bool measure = false;
  
  portENTER_CRITICAL(&timer_mux);
  if (timer_start_measurement) {
    measure = true;
    timer_start_measurement = false;
  }
  portEXIT_CRITICAL(&timer_mux);

  if (measure) {
    measurements_read_and_send();
  }
}
